/*
 * @Author: Jimmy jimmy.jiang@vinno.com
 * @Date: 2022-06-23 20:39:01
 * @LastEditors: Jimmy jimmy.jiang@vinno.com
 * @LastEditTime: 2022-07-01 18:29:24
 * @FilePath: \fis_lib_pdf\lib\pdf_objects\pdf_element.dart
 * @Description: 这是默认设置,请设置`customMade`, 打开koroFileHeader查看配置 进行设置: https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
 */
import 'package:fis_lib_pdf/pdf_objects/report/pdf_imageList.dart';
import 'package:fis_lib_pdf/pdf_objects/report/pdf_paragraph.dart';
import 'package:fis_lib_pdf/pdf_objects/report/report_leaf_element/pdf_leaf_element.dart';
import 'package:flutter/cupertino.dart';
import 'package:pdf/widgets.dart' as pw;
import 'report/pdf_rttable.dart';
import './report/report_helper/report_dic.dart';

abstract class PdfElement {
  ///模板主要列表结构的信息转换为PDF组件列表
  @protected
  pw.Widget generatePDFElement() {
    return pw.Container();
  }

  pw.Widget generateElementByBlockInfo(IBlockElementInfo elementInfo,
      {bool parentIsRTCell = false, bool parentIsTable = false}) {
    pw.Widget ouputWidget = pw.Text("*");
    if (elementInfo is RTTableInfo) {
      var pdfTable = PdfRTTable(elementInfo, parentIsTable: parentIsTable);
      ouputWidget = pdfTable.generatePDFElement();
    }
    if (elementInfo is ParagraphInfo) {
      var pdfParagraph = PdfParagraph(elementInfo);
      ouputWidget =
          pdfParagraph.generatePDFElement(isParentRTCell: parentIsRTCell);
    }
    if (elementInfo is InputImageListInfo) {
      var pdfInputImageList = PdfImageList(elementInfo);
      ouputWidget = pdfInputImageList.generatePDFElementAsync();
    }

    return ouputWidget;
  }

  ///根据模板具体element信息生成对应element的pdf组件
  pw.Widget generateElementByElementType(ElementInfo elementInfo,
      {bool parentElementHasNoContainer = false,
      bool parentIsTable = false,
      bool parentIsRTCell = false}) {
    if (elementInfo is StaticTextInfo) {
      return PdfLeafElement.getTextPdfWidget(
          elementInfo, parentIsTable, parentIsRTCell);
    }
    if (elementInfo is LineInfo) {
      return PdfLeafElement.getLinePdfWidget(elementInfo);
    }
    if (elementInfo is DateTimeInfo) {
      return PdfLeafElement.getDateTimePdfWidget(elementInfo);
    }
    if (elementInfo is InputTextInfo) {
      return PdfLeafElement.getInputTextPdfWidget(
          elementInfo, parentElementHasNoContainer);
    }
    if (elementInfo is SingleSelectedInfo) {
      return PdfLeafElement.getSingleSelectPdfWidget(elementInfo);
    }
    if (elementInfo is MulitiSelectedInfo) {
      return PdfLeafElement.getMultiSelectPdfWidget(elementInfo);
    }
    if (elementInfo is StaticImageInfo) {
      return PdfLeafElement.getStaticImagePdfWidget(elementInfo);
    }
    if (elementInfo is InputImageInfo) {
      return PdfLeafElement.getInputImagePdfWidget(elementInfo);
    }

    return pw.Text("UnKnown- ${elementInfo.elementType!.name}");
  }
}
