import 'package:fis_lib_pdf/pdf_objects/pdf_header.dart';
import 'package:fis_lib_pdf/ultils/pdf_type_enum.dart';
import 'package:fis_lib_report/report_info/report_info.dart';
import 'package:pdf/pdf.dart';
import 'package:pdf/widgets.dart' as pw;

import '../ultils/recources_catch.dart';

import 'pdf_body.dart';
import 'pdf_footer.dart';
import 'report/report_helper/report_dic.dart';

class TextText extends pw.RichText with pw.SpanningWidget {
  TextText(
    String text, {
    pw.TextStyle? style,
    pw.TextAlign? textAlign,
    pw.TextDirection? textDirection,
    bool? softWrap,
    bool tightBounds = false,
    double textScaleFactor = 1.0,
    int? maxLines,
    pw.TextOverflow? overflow,
  }) : super(
          text: pw.TextSpan(text: text, style: style),
          textAlign: textAlign!,
          softWrap: softWrap!,
          tightBounds: tightBounds,
          textDirection: textDirection,
          textScaleFactor: textScaleFactor,
          maxLines: maxLines,
          overflow: overflow,
        );
}

///PDF页面生成器
class PdfPageCreator {
  PdfPageCreator(
      this.reportInfo, this.pdfType, this.defaultFont, this.them, this.format);
  pw.ThemeData them;
  PdfPageFormat format;
  FReportInfo reportInfo;
  PDFTypeEnum pdfType;
  pw.Font defaultFont;

  ///根据传入的具体PDF类型生成对应的PDF页面
  Future<pw.Document> getPage() async {
    var doc = pw.Document(pageMode: PdfPageMode.outlines);
    doc.addPage(_generatePdfPage());
    return doc;
  }

  static Future<pw.Document> getPageFromJPGE(
    List<String> jpges,
    PdfPageFormat format,
  ) async {
    var doc = pw.Document(pageMode: PdfPageMode.outlines);
    for (var j in jpges) {
      var body = _generateJPGReportBody(j);
      var page = pw.Page(
          pageFormat: format.copyWith(
              marginLeft: 0 * PdfPageFormat.cm,
              marginRight: 0 * PdfPageFormat.cm,
              marginTop: 0 * PdfPageFormat.cm,
              marginBottom: 0 * PdfPageFormat.cm),
          build: (pw.Context context) {
            return body;
          });
      doc.addPage(page);
    }
    return doc;
  }

  pw.Page _generatePdfPage() {
    if (pdfType == PDFTypeEnum.report) {
      return pw.MultiPage(
          pageFormat: format.copyWith(
            marginLeft: reportInfo.reportTemplate!.pagePadding!.left! *
                (1.76 / 5) *
                PdfPageFormat.mm,
            marginRight: reportInfo.reportTemplate!.pagePadding!.right! *
                (1.76 / 5) *
                PdfPageFormat.mm,
            marginTop: reportInfo.reportTemplate!.pagePadding!.top! *
                (1.76 / 5) *
                PdfPageFormat.mm,
            marginBottom: reportInfo.reportTemplate!.pagePadding!.bottom! *
                (1.76 / 5) *
                PdfPageFormat.mm,
          ),
          textDirection: pw.TextDirection.ltr,
          crossAxisAlignment: pw.CrossAxisAlignment.stretch,
          footer: _generateFooterPDFElement,
          header: _generateHeaderElement,
          build: _generateBodyElements);
    }

    //当没有指定的pdf类型时的缺省页面
    return pw.Page(build: (context) {
      return pw.Text('No pdf type was set',
          style: pw.TextStyle(
              font: defaultFont, fontSize: 18, fontWeight: pw.FontWeight.bold));
    });
  }

  List<pw.Widget> _generateBodyElements(pw.Context context) {
    List<IBlockElementInfo> blocks = [];
    blocks.addAll(reportInfo.blocks);
    var body = PdfBody(blocks);
    var bodyWidgets = body.generatedPDFElements();
    if (bodyWidgets.isEmpty) {
      bodyWidgets.add(pw.Text(""));
    }
    return bodyWidgets;
  }

  pw.Widget _generateFooterPDFElement(pw.Context context) {
    var pdfFooter = PdfFooter(reportInfo.footers);
    return pdfFooter.getFooter();
  }

  pw.Widget _generateHeaderElement(pw.Context context) {
    var header = PdfHeader(reportInfo.headers);
    return header.getHeader();
  }

  static pw.Widget _generateJPGReportBody(String jpegReport) {
    var jpg = ResourcesCache.getCacheCloudImages(jpegReport);
    return pw.Image(jpg!, fit: pw.BoxFit.fitWidth);
  }
}
