/*
 * @Author: Jimmy jimmy.jiang@vinno.com
 * @Date: 2022-06-23 20:39:01
 * @LastEditors: Jimmy jimmy.jiang@vinno.com
 * @LastEditTime: 2022-07-01 15:42:41
 * @FilePath: \fis_lib_pdf\lib\pdf_objects\report\pdf_imageList.dart
 * @Description: 
 * 
 * Copyright (c) 2022 by Jimmy jimmy.jiang@vinno.com, All Rights Reserved. 
 */
import 'package:fis_lib_pdf/pdf_objects/pdf_element.dart';
import 'package:fis_lib_pdf/pdf_objects/report/report_helper/report_style_helper.dart';
import 'package:fis_lib_pdf/ultils/recources_catch.dart';
import 'package:pdf/pdf.dart';
import 'package:pdf/widgets.dart' as pw;
import './report_helper/report_dic.dart';

///Pdf图片列表组件
class PdfImageList extends PdfElement {
  PdfImageList(this.imageListInfo);
  InputImageListInfo imageListInfo;

  ///根据构造传值来的imageListInfo生成一组包含pdf图片的组件结构。
  pw.Widget generatePDFElementAsync() {
    var childs = <pw.Widget>[];
    var imageListElement = imageListInfo.element as InputImageList;

    var columCount = imageListElement.column;
    var imageWidth = imageListElement.imageWidth;
    var imageHeight = imageListElement.imageHeight;
    var selectedImaged = imageListInfo.selectedImages;
    _generateElementByColumn(
        columCount, imageWidth, imageHeight, childs, selectedImaged);

    var elementHorizontalAlignment =
        imageListInfo.element?.horizontalAlignment!;
    var pwMargin =
        ReportStyleHelper.convertRTMargin(imageListInfo.element?.margin);
    var horAlign =
        ReportStyleHelper.getRowMainAisAlign(elementHorizontalAlignment!);
    var container = pw.Container(
        margin: pwMargin,
        child: pw.Column(mainAxisAlignment: horAlign, children: childs));
    return container;
  }

  void _generateElementByColumn(
      int? columCount,
      double? imageWidth,
      double? imageHeight,
      List<pw.Widget> childs,
      List<String> imageUrls) async {
    var realImages = <pw.MemoryImage>[];
    for (var i in imageUrls) {
      var imageData = ResourcesCache.getCacheCloudImages(i);
      if (imageData != null) {
        realImages.add(imageData);
      }
    }
    _generateImageList(columCount, imageWidth, imageHeight, childs, realImages);
  }

  int _generateImageList(
      int? columCount,
      double? imageWidth,
      double? imageHeight,
      List<pw.Widget> childs,
      List<pw.MemoryImage> values) {
    int index = 0;
    List<pw.Widget> rowChilds = [];
    for (var element in values) {
      if (rowChilds.isNotEmpty) {
        rowChilds.add(pw.SizedBox(width: 2));
      }
      rowChilds.add(
        pw.Container(
          width: imageWidth,
          height: imageHeight,
          color: const PdfColor(0, 0, 0, 1),
          child: pw.Center(
            child: pw.Image(element,
                width: imageWidth, height: imageHeight, fit: pw.BoxFit.contain),
          ),
        ),
      );

      index++;
      if (index % (columCount!) == 0 || index == values.length) {
        if (childs.isNotEmpty) {
          childs.add(pw.SizedBox(height: 2));
        }
        childs.add(pw.Row(
            mainAxisAlignment: pw.MainAxisAlignment.start,
            children: rowChilds.toList()));

        rowChilds = [];
      }
    }
    return 0;
  }
}
