/*
 * @Author: Jimmy jimmy.jiang@vinno.com
 * @Date: 2022-06-23 20:39:01
 * @LastEditors: Jimmy jimmy.jiang@vinno.com
 * @LastEditTime: 2022-07-01 13:20:20
 * @FilePath: \fis_lib_pdf\lib\pdf_objects\report\pdf_RTTable.dart
 * @Description: 这是默认设置,请设置`customMade`, 打开koroFileHeader查看配置 进行设置: https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
 */
import 'dart:math';
import 'package:fis_lib_pdf/pdf_objects/pdf_element.dart';
import 'package:fis_lib_pdf/pdf_objects/report/report_helper/report_style_helper.dart';
import 'package:fis_lib_pdf/table_data_helper.dart';
import 'package:pdf/widgets.dart' as pw;
import './report_helper/report_dic.dart';

class PdfRTTable extends PdfElement {
  PdfRTTable(RTTableInfo rtTableInfo, {this.parentIsTable = false}) {
    _rtTableInfo = TableDataHelper.removeEmptyCells(rtTableInfo);
  }
  late RTTableInfo _rtTableInfo;
  final bool parentIsTable;

  @override
  pw.Widget generatePDFElement() {
    if (_rtTableInfo.cells == null) {
      return pw.Container();
    }
    return pw.Container(
      margin: parentIsTable ? null : const pw.EdgeInsets.only(top: 5),
      child: _generateChild(),
    );
  }

  /// 生成元素
  pw.Widget _generateChild() {
    if (_rtTableInfo.cells == null || _rtTableInfo.cells!.isEmpty) {
      return pw.Container();
    }
    List<BaseGrid> allGrids = [];
    for (MapEntry<CellPostion, RTCellInfo> cell
        in _rtTableInfo.cells!.entries) {
      CellPostion pos = cell.key;
      RTCellInfo cellInfo = cell.value;
      RTCell cellElement = cellInfo.element as RTCell;
      List<pw.Widget> cellContainerChildren = <pw.Widget>[];
      for (var element in cellInfo.blocks!) {
        var cellChild = generateElementByBlockInfo(element,
            parentIsRTCell: true, parentIsTable: true);
        cellContainerChildren.add(cellChild);
      }
      bool multiWidgetsInCell = cellContainerChildren.length > 1;
      pw.Widget cellWidgets;
      if (cellContainerChildren.isEmpty) {
        cellWidgets = pw.Container();
      } else {
        cellWidgets = multiWidgetsInCell
            ? pw.Wrap(
                alignment: ReportStyleHelper.getWrapAlignment(
                    cellElement.horizontalAlignment!),
                crossAxisAlignment: ReportStyleHelper.getWrapCrossAlignment(
                    cellElement.verticalAlignment!),
                children: cellContainerChildren)
            : pw.Align(
                alignment: ReportStyleHelper.getChildAlignment(cellElement),
                child: cellContainerChildren[0]);
      }
      pw.Container cellContainer = pw.Container(
        constraints: pw.BoxConstraints(
          minWidth: _getCellWidth(pos),
          maxWidth: _getCellWidth(pos),
          minHeight: _getCellHeight(pos),
        ),
        // width: _getCellWidth(pos),
        // height: _getCellHeight(pos),
        decoration: parentIsTable
            ? ReportStyleHelper.getAllBlackBorderDecoration(cellElement)
            : ReportStyleHelper.getPDFCellBorderBox(cellElement),
        child: cellWidgets,
      );

      allGrids.add(BaseGrid(
          pos.row!, pos.column!, pos.rowSpan!, pos.columnSpan!, cellContainer));
    }

    BaseGrid? minGrid;

    // 防死循环缓存
    String uniKey = "";
    // 重复循环次数
    int loopCount = 0;

    /// 先找出最小单元格，从列表中移除，然后在剩余的单元格中找出与最小单元格相邻的单元格，合并
    while ((minGrid = _getMinGrid(allGrids)) != null) {
      String newUniKey =
          allGrids.map((e) => e.row.toString() + e.column.toString()).join();
      if (uniKey == newUniKey) {
        loopCount++;
      } else {
        uniKey = newUniKey;
        loopCount = 0;
      }
      if (loopCount > 5) {
        loopCount = 0;
        break;
      }
      allGrids.remove(minGrid);
      List<BaseGrid> gridsToMerge = [];
      while (true) {
        gridsToMerge = _getGridsCanMerge(minGrid!, allGrids);
        if (gridsToMerge.isEmpty) {
          break;
        }
        for (var g in gridsToMerge) {
          allGrids.remove(g);
        }
        minGrid = _mergeGrids(minGrid, gridsToMerge);
      }
      allGrids.add(minGrid);
      if (allGrids.length == 1) {
        break;
      }
    }
    return allGrids[0].child;
  }

  /// 获取单元格宽度
  double _getCellWidth(CellPostion pos) {
    double width = 0;
    int column = pos.column ?? 0;
    int columnSpan = pos.columnSpan ?? 1;
    for (int i = column; i < column + columnSpan; i++) {
      width += _getWidthFromColumnDefinitions(i);
    }
    return width;
  }

  double _getWidthFromColumnDefinitions(int columnIndex) {
    if (columnIndex >= _rtTableInfo.columnDefinitions!.length) {
      return _rtTableInfo.columnDefinitions!.last.width!;
    }
    return _rtTableInfo.columnDefinitions![columnIndex].width!;
  }

  /// 获取单元格高度
  double _getCellHeight(CellPostion pos) {
    double height = 0;
    int row = pos.row ?? 0;
    int rowSpan = pos.rowSpan ?? 1;
    for (int i = row; i < row + rowSpan; i++) {
      height += _getHeightFromRowDefinitions(i);
    }
    return height;
  }

  double _getHeightFromRowDefinitions(int rowIndex) {
    if (rowIndex >= _rtTableInfo.rowDefinitions!.length) {
      return _rtTableInfo.rowDefinitions!.last.height!;
    }
    return _rtTableInfo.rowDefinitions![rowIndex].height!;
  }

  /// 合并单元格
  BaseGrid _mergeGrids(BaseGrid baseGrid, List<BaseGrid> gridsToMerge) {
    int row = baseGrid.row;
    int column = baseGrid.column;
    int rowSpan = baseGrid.rowSpan;
    int columnSpan = baseGrid.columnSpan;
    bool isHorizontal = row == gridsToMerge[0].row;
    List<BaseGrid> allGrids = [baseGrid, ...gridsToMerge];
    pw.Widget child;
    List<pw.Widget> children = [];
    // 横向合并
    if (isHorizontal) {
      /// 根据列号排序
      allGrids.sort((a, b) => a.column.compareTo(b.column));
      for (BaseGrid g in allGrids) {
        if (g.child is pw.Row) {
          children.addAll((g.child as pw.Row).children);
        } else {
          children.add(g.child);
        }
      }
      child = pw.Row(
          children: children, mainAxisAlignment: pw.MainAxisAlignment.center);
    } else {
      /// 根据行号排序
      allGrids.sort((a, b) => a.row.compareTo(b.row));
      for (BaseGrid g in allGrids) {
        if (g.child is pw.Column) {
          children.addAll((g.child as pw.Column).children);
        } else {
          children.add(g.child);
        }
      }
      child = pw.Column(
        children: children,
      );
    }

    for (var g in gridsToMerge) {
      if (row == g.row) {
        columnSpan += g.columnSpan;
      } else {
        rowSpan += g.rowSpan;
      }
      row = min(g.row, row);
      column = min(g.column, column);
    }
    return BaseGrid(row, column, rowSpan, columnSpan, child);
  }

  /// 找出最小单元格
  BaseGrid? _getMinGrid(List<BaseGrid> grids) {
    if (grids.isEmpty) {
      return null;
    }
    grids.sort((a, b) => a.areaSize.compareTo(b.areaSize));
    return grids[0];
  }

  /// 获取可以合并的单元格 (可能有多个，但上下/左右只能选一侧，优先左右)
  List<BaseGrid> _getGridsCanMerge(BaseGrid grid, List<BaseGrid> grids) {
    List<BaseGrid> verticalGrids = <BaseGrid>[];
    List<BaseGrid> horizontalGrids = <BaseGrid>[];
    for (var g in grids) {
      if (_haveSameEdge(grid, g)) {
        if (g.row == grid.row) {
          horizontalGrids.add(g);
        } else {
          verticalGrids.add(g);
        }
      }
    }
    if (horizontalGrids.isNotEmpty) {
      return horizontalGrids;
    } else {
      return verticalGrids;
    }
  }

  /// 判断两个 Grid 是否相邻且有公共边
  bool _haveSameEdge(BaseGrid g1, BaseGrid g2) {
    if (g1.row == g2.row && g1.rowSpan == g2.rowSpan) {
      return g1.column + g1.columnSpan == g2.column ||
          g2.column + g2.columnSpan == g1.column;
    } else if (g1.column == g2.column && g1.columnSpan == g2.columnSpan) {
      return g1.row + g1.rowSpan == g2.row || g2.row + g2.rowSpan == g1.row;
    }
    return false;
  }
}

/// 基础单元
class BaseGrid {
  int row;
  int column;
  int rowSpan;
  int columnSpan;
  int get areaSize => rowSpan * columnSpan; // 单元格面积 (粗略) rowSpan * columnSpan
  pw.Widget child;
  BaseGrid(this.row, this.column, this.rowSpan, this.columnSpan, this.child);
}
