import 'dart:async';
import 'dart:convert';
import 'package:fis_lib_pdf/report_render.dart';
import 'package:fis_lib_pdf/ultils/pdf381_wrapper/pdf381_preview_ext.dart';
import 'package:fis_lib_pdf/ultils/pdf_access_type.dart';
import 'package:fis_lib_pdf/ultils/pdf_type_enum.dart';
import 'package:fis_lib_pdf/ultils/recources_catch.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:pdf/pdf.dart';
import 'package:pdf/widgets.dart' as pw;
import 'package:fis_lib_print/printing.dart';
import 'package:fis_lib_report/report_info/report_info.dart';
import 'package:fis_lib_report/report/report_template_document.dart';
import 'package:fis_common/env/env.dart';
import 'ultils/pdf_render_factory.dart';
import 'package:fis_ui/index.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_common/js_plateform/js_platform.dart';

class PDFConfig {
  static bool demoMode = false;
}

///PDF渲染容器
class FPDFRenderContainer extends StatefulWidget implements FWidget {
  final PDFTypeEnum pdfTypeEnum;
  final bool demoMode;
  final String fileName;
  final PDFAccessTypeEnum pdfAccessTypeEnum;
  final Future<bool> Function(List<int> data, String fileName)? pdfExporter;
  final Future<bool> Function(String fileName, Uint8List file)? cacheSave;
  final Future<bool> Function()? webSideShare;
  final void Function()? onClose;
  final Future<Uint8List?> Function(String fileName)? cacheGet;
  final bool useActionBar;
  final bool fromEditor;
  final List<String>? imageOrFileUrls;
  final List<SelfPdfPreviewAction> customAction;
  final double? dpi;
  final Widget? loadingWidget;
  final bool allowPrinting;
  final bool allowSharing;
  final bool isVital;

  const FPDFRenderContainer({
    Key? key,
    required this.pdfTypeEnum,
    required this.demoMode,
    required this.pdfAccessTypeEnum,
    required this.pdfExporter,
    required this.cacheSave,
    required this.cacheGet,
    required this.fromEditor,
    this.webSideShare,
    this.fileName = "document.pdf",
    this.useActionBar = true,
    this.imageOrFileUrls,
    this.onClose,
    this.customAction = const [],
    this.dpi = 300,
    this.loadingWidget,
    this.allowPrinting = true,
    this.allowSharing = true,
    this.isVital = false,
  }) : super(key: key);

  @override
  FPDFRenderContainerState createState() {
    PDFConfig.demoMode = demoMode;
    return FPDFRenderContainerState();
  }
}

class FPDFRenderContainerState extends State<FPDFRenderContainer>
    with SingleTickerProviderStateMixin {
  bool isinit = false;
  DateTime? lastOperationTime;

  ///这里是demo测试数据
  var mockedTemplate = <String>[
    'assets/fonts/jimmyCes.json',
  ];
  var currentTemplateIndex = 0;
  PrintingInfo? printingInfo;

  @override
  void initState() {
    super.initState();
    _init();
  }

  @override
  void dispose() {
    print("PDF contaner is disposed");
    ResourcesCache.clearCachedPDF();
    super.dispose();
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
  }

  Future<void> _init() async {
    await _initReportInfo();
    final info = await Printing.info();
    ResourcesCache.currentPDFFileKey = widget.fileName;

    setState(() {
      printingInfo = info;
    });
  }

  ///默认尺寸
  double? _scale = 960;
  Future<void> _saveAsFile(
    BuildContext context,
    LayoutCallback build,
    PdfPageFormat pageFormat,
  ) async {
    final bytes =
        ResourcesCache.getCachedPDFBinarys() ?? await build(pageFormat);

    ResourcesCache.getCachedPDFBinarys() ?? await build(pageFormat);
    var byteList = bytes.toList();
    if (_useFileExporertUI()) {
      await widget.pdfExporter!.call(byteList, widget.fileName);
    } else {
      await Printing.sharePdf(
        bytes: bytes,
        filename: widget.fileName,
      );
    }
  }

  bool _useFileExporertUI() {
    final platform = FPlatform.current;
    return platform == FPlatformEnum.webOnWin ||
        platform == FPlatformEnum.webOnMac;
  }

  Decoration? _getPdfDecoration() {
    return const BoxDecoration(color: Colors.white);
  }

  //直接下载
  Future<void> downloadDirectly(String fileName, {bool isPrint = false}) async {
    var bytes =
        ResourcesCache.getCachedPDFBinarys() ?? await _getPDFRenderMethod();
    if (lastOperationTime != null) {
      var operationPeriod = DateTime.now().difference(lastOperationTime!);
      var second = operationPeriod.inSeconds;
      //防止频繁下载
      if (second < 1) {
        return;
      }
    }
    lastOperationTime = DateTime.now();
    if (isPrint) {
      var base64 = base64Encode(bytes);
      JSPlateForm.printPdf(base64);
      return;
    }
    if (_useFileExporertUI()) {
      await widget.pdfExporter!.call(bytes, fileName);
      return;
    }
    await Printing.sharePdf(
      bytes: bytes,
      filename: fileName,
    );
  }

  @override
  Widget build(BuildContext context) {
    ResourcesCache.currentPDFFileKey = widget.fileName;
    pw.RichText.debug = true;
    ResourcesCache.cacheGet = widget.cacheGet;
    ResourcesCache.cacheSave = widget.cacheSave;

    final actions = <SelfPdfPreviewAction>[
      SelfPdfPreviewAction(
        icon: const Icon(Icons.save),
        onPressed: _saveAsFile,
        content: PDFConfig.demoMode ? "save" : i18nBook.common.save.t,
      ),
      if (widget.onClose != null &&
          widget.fromEditor &&
          (FPlatform.isWindows || FPlatform.isMacOS)) ...[
        SelfPdfPreviewAction(
          content: PDFConfig.demoMode ? "back" : i18nBook.common.back.t,
          icon: const Icon(Icons.undo_sharp),
          onPressed: (_, onPressed, _format) async {
            widget.onClose!.call();
          },
        ),
      ],
      ...widget.customAction,
    ];

    if (widget.pdfAccessTypeEnum == PDFAccessTypeEnum.printDirectly) {
      downloadDirectly(widget.fileName, isPrint: true);
      return Container();
    }
    if (widget.pdfAccessTypeEnum == PDFAccessTypeEnum.donwnloadDirectly) {
      downloadDirectly(widget.fileName);
      return Container();
    }

    var isReleaseMobile = (!widget.demoMode && kIsMobile);
    return FStack(
      children: [
        FScaffold(
          body: Pdf381PreviewExt(
            maxPageWidth:
                isReleaseMobile ? MediaQuery.of(context).size.width : _scale,
            onZoom: _onZoomInOrOut,
            build: (format) =>
                PDFRenderFactory.GetPDFRenderByPDFTypeEnum(widget.pdfTypeEnum)(
              format,
              widget.imageOrFileUrls,
            ),
            usesrDefinedActions: isReleaseMobile ? null : actions,
            scrollViewDecoration: _getPdfDecoration(),
            useActions: widget.useActionBar,
            allowSharing: !widget.fromEditor && widget.allowSharing,
            onWebSideShare: widget.webSideShare,
            isReleaseMobile: isReleaseMobile,
            loadingWidget: widget.loadingWidget,
            isVital: widget.isVital,
            allowPrinting: widget.allowPrinting,
            dpi: widget.pdfTypeEnum == PDFTypeEnum.pdfJpeg
                ? null // pdfJpeg 是来自老平台的数据，不需要设置 dpi
                : widget.dpi, // 用于解决 PC 客户端下字体模糊的问题
          ),
        )
      ],
    );
  }

  Future _initReportInfo() async {
    if (widget.demoMode) {
      final jsonStr =
          await rootBundle.loadString(mockedTemplate[currentTemplateIndex]);

      final reportMap = jsonDecode(jsonStr);

      final template = ReportTemplateDocument.fromJson(reportMap);

      if (!isinit) {
        FReportInfo.instance.init(template, DateTime.now(), "",
            revoke: "xxx", selectEntry: "3333", selectImageHint: "adsasd");
      }
    }
    isinit = true;
  }

  void _onZoomInOrOut(PdfZoomMode mode) {
    final newScale =
        mode == PdfZoomMode.pdfZoomIn ? _scale! + 100.0 : _scale! - 100.0;
    if (newScale < 200.0) {
      return;
    }
    setState(() {
      _scale = newScale;
    });
  }

  Future<Uint8List> _getPDFRenderMethod() async {
    if (widget.pdfTypeEnum == PDFTypeEnum.pdfJpeg) {
      return await generateByGivenPDFJpges(
          PdfPageFormat.a4, widget.imageOrFileUrls);
    } else if (widget.pdfTypeEnum == PDFTypeEnum.pdfBytes) {
      return await generateByGivenPDFBytes(
          PdfPageFormat.a4, widget.imageOrFileUrls);
    } else {
      /// 从内存内的 报告内容生成 并缓存 可以用于下次下载
      return await generateReport(PdfPageFormat.a4, null);
    }
  }
}
