import 'dart:typed_data';

import 'package:fis_lib_pdf/ultils/pdf_type_enum.dart';
import 'package:flutter/material.dart';
import 'package:pdf/pdf.dart';
import 'package:pdf/widgets.dart' as pw;

import 'package:fis_lib_pdf/ultils/recources_catch.dart';
import 'package:fis_lib_report/report_info/report_info.dart';

import 'pdf_objects/pdf_page.dart';

import 'package:http/http.dart' as http;

/// 根据报告页面格式生成报告pdf
Future<Uint8List> generateReport(
    PdfPageFormat pageFormat, List<String>? imageOrFileUrls) async {
  final theme = pw.ThemeData.base();
  await ResourcesCache.cacheDefaultFont("defaultCN");
  await ResourcesCache.cacheAllCloudImages(FReportInfo.instance);

  var pdfPage = PdfPageCreator(FReportInfo.instance, PDFTypeEnum.report,
      ResourcesCache.pdfCustomFonts["defaultCN"]!, pw.ThemeData(), pageFormat);

  var document = await pdfPage.getPage();

  var pdf = await _getPDFBinary(document);

  return pdf;
}

/// 根据报告页面格式生成报告pdf
Future<Uint8List> generateByGivenPDFBytes(
    PdfPageFormat pageFormat, List<String>? imageOrFileUrls) async {
  Uint8List bytes = Uint8List(0);
  if (imageOrFileUrls != null && imageOrFileUrls.isNotEmpty) {
    String fileUrl = imageOrFileUrls[0];
    var cachedDoc = ResourcesCache.getCachedPDFBinarys();
    if (cachedDoc != null) {
      return cachedDoc;
    }
    try {
      // getCachedPDFBinarys
      bytes = await _downloadFile(fileUrl);
      ResourcesCache.cachePDFBinarys(bytes);
    } catch (e) {
      debugPrint(e.toString());
    }
  }

  return bytes;
}

Future<Uint8List> generateByGivenPDFJpges(
    PdfPageFormat pageFormat, List<String>? imageOrFileUrls) async {
  await ResourcesCache.cacheNetworkResources(imageOrFileUrls!);
  var pdfPage =
      await PdfPageCreator.getPageFromJPGE(imageOrFileUrls, pageFormat);
  return await _getPDFBinary(pdfPage);
}

Future<Uint8List> _getPDFBinary(pw.Document document) async {
  var data = document.save();
  var realDocData = await data;
  var cachedDoc = ResourcesCache.getCachedPDFBinarys();
  if (cachedDoc != null) {
    return cachedDoc;
  } else {
    ResourcesCache.cachePDFBinarys(realDocData);
    return data;
  }
}

Future<Uint8List> _downloadFile(String fileUrl) async {
  var response = await http.get(Uri.parse(fileUrl));
  if (response.statusCode == 200) {
    return response.bodyBytes;
  } else {
    throw Exception('Failed to download file');
  }
}
