/*
 * Copyright (C) 2017, David PHAM-VAN <dev.nfet.net@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///此代码为基于pdf3.8.1通过复制代码形式做的硬扩展，本质上属于包资源，不属于FIS代码范畴
///如有疑问可参考pdf_preview
///为实现pdf_preview放大缩小功能，对PDF做了ZoomIn，ZoomOut的回调包装：

import 'package:fis_lib_pdf/pdf_render_container.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:pdf/pdf.dart';
import 'package:fis_ui/interface/interactive_container.dart';
import 'package:fis_lib_print/printing.dart';
// ignore: implementation_imports
import 'package:fis_lib_print/src/preview/controller.dart';
import 'package:fis_ui/index.dart';
import 'package:fis_i18n/i18n.dart';

enum PdfZoomMode { pdfZoomIn, pdfZoomOut }

typedef OnZoomFunction = void Function(PdfZoomMode o);

class PdfZoomAction extends StatelessWidget with PdfPreviewActionBounds {
  PdfZoomAction({Key? key, Widget? icon, this.zoomMode, this.onZoom})
      : icon = icon ?? const Icon(Icons.add),
        super(key: key);
  final Widget icon;
  final OnZoomFunction? onZoom;
  final PdfZoomMode? zoomMode;
  @override
  Widget build(BuildContext context) {
    return SelfPdfPreviewAction(
      key: childKey,
      icon: icon,
      onPressed: _zoom,
      content: _getButtonContent(),
    );
  }

  String _getButtonContent() {
    if (zoomMode == PdfZoomMode.pdfZoomIn) {
      return PDFConfig.demoMode ? "Zoomin" : i18nBook.common.zoomIn.t;
    } else {
      return PDFConfig.demoMode ? "Zoomout" : i18nBook.common.zoomOut.t;
    }
  }

  Future<void> _zoom(BuildContext context, LayoutCallback build,
      PdfPageFormat pageFormat) async {
    onZoom?.call(zoomMode!);
  }
}

class SelfPdfPrintAction extends StatelessWidget {
  const SelfPdfPrintAction({
    Key? key,
    Widget? icon,
    String? jobName,
    this.onPrinted,
    this.onPrintError,
    this.dynamicLayout = true,
    this.usePrinterSettings = false,
  })  : icon = icon ?? const Icon(Icons.print),
        jobName = jobName ?? 'Document',
        super(key: key);

  final Widget icon;

  final String jobName;

  /// Request page re-layout to match the printer paper and margins.
  /// Mitigate an issue with iOS and macOS print dialog that prevent any
  /// channel message while opened.
  final bool dynamicLayout;

  /// Set [usePrinterSettings] to true to use the configuration defined by
  /// the printer. May not work for all the printers and can depend on the
  /// drivers. (Supported platforms: Windows)
  final bool usePrinterSettings;

  /// Called if the user prints the pdf document
  final VoidCallback? onPrinted;

  /// Called if an error creating the Pdf occurred
  final void Function(dynamic error)? onPrintError;

  @override
  Widget build(BuildContext context) {
    return SelfPdfPreviewAction(
      icon: icon,
      onPressed: _print,
      content: PDFConfig.demoMode ? "print" : i18nBook.remedical.print.t,
    );
  }

  Future<void> _print(
    BuildContext context,
    LayoutCallback build,
    PdfPageFormat pageFormat,
  ) async {
    final data = PdfPreviewController.of(context);

    try {
      final result = await Printing.layoutPdf(
        onLayout: build,
        name: jobName,
        format: data.actualPageFormat,
        dynamicLayout: dynamicLayout,
        usePrinterSettings: usePrinterSettings,
      );

      if (result) {
        onPrinted?.call();
      }
    } catch (exception, stack) {
      InformationCollector? collector;

      assert(() {
        collector = () sync* {
          yield StringProperty('PageFormat', data.actualPageFormat.toString());
        };
        return true;
      }());

      FlutterError.reportError(FlutterErrorDetails(
        exception: exception,
        stack: stack,
        library: 'printing',
        context: ErrorDescription('while printing a PDF'),
        informationCollector: collector,
      ));

      onPrintError?.call(exception);
    }
  }
}

class SelfPdfPreviewAction extends StatelessWidget
    implements FInteractiveContainer {
  /// Represents an icon to add to [PdfPreview]
  const SelfPdfPreviewAction(
      {Key? key,
      required this.icon,
      required this.onPressed,
      required this.content,
      required})
      : super(key: key);

  /// The icon to display
  final Widget icon;

  /// The callback called when the user tap on the icon
  final OnPdfPreviewActionPressed? onPressed;

  final String content;

  @override
  Widget build(BuildContext context) {
    return ElevatedButton.icon(
      onPressed: onPressed == null ? null : () => pressed(context),
      label: FText(
        content,
      ),
      icon: icon,
    );
  }

  Future<void> pressed(BuildContext context) async {
    final data = PdfPreviewController.of(context);
    onPressed!(context, data.buildDocument, data.pageFormat);
  }

  @override
  // TODO: implement pageName
  String get pageName => "Preview";
}

/// Flutter widget that uses the rasterized pdf pages to display a document.
class Pdf381PreviewExt extends StatefulWidget implements FWidget {
  /// Show a pdf document built on demand
  const Pdf381PreviewExt({
    Key? key,
    required this.build,
    required this.onZoom,
    this.initialPageFormat,
    this.allowPrinting = true,
    this.allowSharing = true,
    this.maxPageWidth,
    this.canChangePageFormat = true,
    this.canChangeOrientation = true,
    this.canDebug = true,
    this.actions,
    this.usesrDefinedActions,
    this.pageFormats = _defaultPageFormats,
    this.onError,
    this.onPrinted,
    this.onPrintError,
    this.onShared,
    this.scrollViewDecoration,
    this.pdfPreviewPageDecoration,
    this.pdfFileName,
    this.useActions = true,
    this.pages,
    this.dynamicLayout = true,
    this.shareActionExtraBody,
    this.shareActionExtraSubject,
    this.shareActionExtraEmails,
    this.previewPageMargin,
    this.padding,
    this.shouldRepaint = false,
    this.loadingWidget,
    this.onPageFormatChanged,
    this.dpi,
    this.onWebSideShare,
    this.isReleaseMobile = false,
    this.isVital = false,
  }) : super(key: key);

  static const _defaultPageFormats = <String, PdfPageFormat>{
    'A4': PdfPageFormat.a4,
    'Letter': PdfPageFormat.letter,
  };

  /// Called when a pdf document is needed
  final LayoutCallback build;

  /// Pdf page format asked for the first display
  final PdfPageFormat? initialPageFormat;
  final OnZoomFunction? onZoom;

  final Future<bool> Function()? onWebSideShare;

  /// Add a button to print the pdf document
  final bool allowPrinting;

  /// Add a button to share the pdf document
  final bool allowSharing;

  /// Allow disable actions
  final bool useActions;

  /// Maximum width of the pdf document on screen
  final double? maxPageWidth;

  /// Add a drop-down menu to choose the page format
  final bool canChangePageFormat;

  /// Add a switch to change the page orientation
  final bool canChangeOrientation;

  /// Add a switch to show debug view
  final bool canDebug;

  /// Additional actions to add to the widget
  final List<Widget>? actions;

  /// 用户自定义Action
  final List<Widget>? usesrDefinedActions;

  /// List of page formats the user can choose
  final Map<String, PdfPageFormat> pageFormats;

  /// Widget to display if the PDF document cannot be displayed
  final Widget Function(BuildContext context, Object error)? onError;

  /// Called if the user prints the pdf document
  final void Function(BuildContext context)? onPrinted;

  /// Called if an error creating the Pdf occurred
  final void Function(BuildContext context, dynamic error)? onPrintError;

  /// Called if the user shares the pdf document
  final void Function(BuildContext context)? onShared;

  /// Decoration of scrollView
  final Decoration? scrollViewDecoration;

  /// Decoration of PdfPreviewPage
  final Decoration? pdfPreviewPageDecoration;

  /// Name of the PDF when sharing. It must include the extension.
  final String? pdfFileName;

  /// Pages to display. Default will display all the pages.
  final List<int>? pages;

  /// Request page re-layout to match the printer paper and margins.
  /// Mitigate an issue with iOS and macOS print dialog that prevent any
  /// channel message while opened.
  final bool dynamicLayout;

  /// email subject when email application is selected from the share dialog
  final String? shareActionExtraSubject;

  /// extra text to share with Pdf document
  final String? shareActionExtraBody;

  /// list of email addresses which will be filled automatically if the email application
  /// is selected from the share dialog.
  /// This will work only for Android platform.
  final List<String>? shareActionExtraEmails;

  /// margin for the document preview page
  ///
  /// defaults to [EdgeInsets.only(left: 20, top: 8, right: 20, bottom: 12)],
  final EdgeInsets? previewPageMargin;

  /// padding for the pdf_preview widget
  final EdgeInsets? padding;

  /// Force repainting the PDF document
  final bool shouldRepaint;

  /// Custom loading widget to use that is shown while PDF is being generated.
  /// If null, a [CircularProgressIndicator] is used instead.
  final Widget? loadingWidget;

  /// The page format has changed
  final ValueChanged<PdfPageFormat>? onPageFormatChanged;

  /// The rendering dots per inch resolution
  /// If not provided, this value is calculated.
  final double? dpi;

  final bool isReleaseMobile;

  final bool isVital;

  @override
  _PdfPreviewState createState() => _PdfPreviewState();
}

class _PdfPreviewState extends State<Pdf381PreviewExt> {
  final previewWidget = GlobalKey<PdfPreviewCustomState>();
  late PdfPreviewData previewData;

  /// Printing subsystem information
  PrintingInfo? info;
  double? maxWidth = 0.0;
  var infoLoaded = false;

  PdfPageFormat computeActualPageFormat() {
    var format = previewData.pageFormat;
    final pages = previewWidget.currentState?.pages ?? const [];
    final dpi = previewWidget.currentState?.dpi ?? PdfPageFormat.inch;

    if (!widget.canChangePageFormat && pages.isNotEmpty) {
      format = PdfPageFormat(
        pages.first.width * PdfPageFormat.inch / dpi,
        pages.first.height * PdfPageFormat.inch / dpi,
        marginAll: 5 * PdfPageFormat.mm,
      );
    }

    return format;
  }

  @override
  void initState() {
    previewData = PdfPreviewData(
      buildDocument: widget.build,
      pageFormats: widget.pageFormats.isNotEmpty
          ? widget.pageFormats
          : Pdf381PreviewExt._defaultPageFormats,
      initialPageFormat: widget.initialPageFormat,
      onComputeActualPageFormat: computeActualPageFormat,
    );

    previewData.addListener(() {
      if (mounted) {
        setState(() {});
      }
      widget.onPageFormatChanged?.call(previewData.pageFormat);
    });

    super.initState();
  }

  @override
  void dispose() {
    previewData.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant Pdf381PreviewExt oldWidget) {
    if (oldWidget.build != widget.build ||
        widget.shouldRepaint ||
        widget.pageFormats != oldWidget.pageFormats) {
      previewData = PdfPreviewData(
        buildDocument: widget.build,
        pageFormats: widget.pageFormats.isNotEmpty
            ? widget.pageFormats
            : Pdf381PreviewExt._defaultPageFormats,
        initialPageFormat: previewData.pageFormat,
        onComputeActualPageFormat: computeActualPageFormat,
      );
      setState(() {});
    }
    super.didUpdateWidget(oldWidget);
  }

  @override
  void didChangeDependencies() {
    if (!infoLoaded) {
      infoLoaded = true;
      Printing.info().then((PrintingInfo _info) {
        setState(() {
          info = _info;
        });
      });
    }

    super.didChangeDependencies();
  }

  _onZoom(PdfZoomMode mode) {
    widget.onZoom?.call(mode);
  }

  Future<void> _onWebSideShare(
    BuildContext context,
    LayoutCallback build,
    PdfPageFormat pageFormat,
  ) async {
    await widget.onWebSideShare!.call();
  }

  @override
  Widget build(BuildContext context) {
    final theme = Theme.of(context);
    final iconColor = theme.primaryIconTheme.color ?? Colors.white;

    final actions = <Widget>[];

    if (widget.useActions && widget.allowPrinting && info?.canPrint == true) {
      var print = SelfPdfPrintAction(
        jobName: widget.pdfFileName,
        dynamicLayout: widget.dynamicLayout,
        onPrinted:
            widget.onPrinted == null ? null : () => widget.onPrinted!(context),
        onPrintError: widget.onPrintError == null
            ? null
            : (dynamic error) => widget.onPrintError!(context, error),
      );
      actions.add(print);
    }

    if (widget.useActions && widget.allowSharing && info?.canShare == true) {
      if (!kIsWeb && !widget.isVital) {
        actions.add(PdfShareAction(
          filename: widget.pdfFileName,
          icon: const Icon(Icons.share),
          onShared:
              widget.onShared == null ? null : () => widget.onShared!(context),
        ));
      } else {
        actions.add(SelfPdfPreviewAction(
          icon: const Icon(Icons.share),
          onPressed: _onWebSideShare,
          content: PDFConfig.demoMode ? "share" : i18nBook.remedical.share.t,
        ));
      }
    }
    if (!widget.isReleaseMobile) {
      actions.add(PdfZoomAction(
          icon: const Icon(Icons.zoom_in),
          zoomMode: PdfZoomMode.pdfZoomIn,
          onZoom: _onZoom));
      actions.add(PdfZoomAction(
          icon: const Icon(Icons.zoom_out),
          zoomMode: PdfZoomMode.pdfZoomOut,
          onZoom: _onZoom));
    }
    if (widget.useActions) {
      widget.usesrDefinedActions?.forEach((element) {
        actions.add(element);
      });
    }

    maxWidth = widget.maxPageWidth;

    widget.actions?.forEach(actions.add);

    assert(() {
      return true;
    }());

    return PdfPreviewController(
      data: previewData,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          if (actions.isNotEmpty)
            IconTheme.merge(
              data: const IconThemeData(color: Colors.blue, size: 28),
              child: Material(
                elevation: 4,
                color: Colors.grey.shade50,
                child: SizedBox(
                  width: double.infinity,
                  child: Container(
                    padding: const EdgeInsets.fromLTRB(0, 13, 0, 5),
                    child: Wrap(
                      alignment: WrapAlignment.center,
                      children: actions,
                      spacing: 20,
                    ),
                  ),
                ),
              ),
            ),
          Expanded(
            child: Builder(builder: (context) {
              final controller = PdfPreviewController.listen(context);
              return PdfPreviewCustom(
                key: previewWidget,
                build: controller.buildDocument,
                loadingWidget: widget.loadingWidget,
                maxPageWidth: maxWidth,
                onError: widget.onError,
                padding: widget.padding,
                pageFormat: controller.pageFormat,
                pages: widget.pages,
                pdfPreviewPageDecoration: widget.pdfPreviewPageDecoration,
                previewPageMargin: widget.previewPageMargin,
                scrollViewDecoration: widget.scrollViewDecoration,
                shouldRepaint: widget.shouldRepaint,
                dpi: widget.dpi,
              );
            }),
          )
        ],
      ),
    );
  }
}
